import sys
from PyQt4.QtCore import Qt, QRectF as Rect, QPointF as Point, QLineF as Line
from PyQt4.QtGui import (QApplication, QGraphicsScene, QGraphicsView,
                         QColor, QGraphicsItem, QGraphicsEllipseItem,
                         QGraphicsPixmapItem, QPixmap, QGraphicsEllipseItem,
                         QGraphicsWidget)
import numpy as np
from itertools import izip
from csc.util.persist import load_pickle
from csc.divisi.tensor import data, DenseTensor
from csc.divisi.labeled_view import LabeledView

# This initializes Qt, and nothing works without it. Even though we
# don't use the "app" variable until the end.
app = QApplication(sys.argv)

class QImageWidget(QGraphicsWidget):
    def __init__(self, image):
        '''
        A QGraphicsItem that draws a QImage.
        '''
        super(QImageWidget, self).__init__()
        self.image = image
        
    def paint(self, painter, option, widget):
        painter.drawImage(0, 0, self.image)


class SVDPoint(QGraphicsEllipseItem):
    __slots__ = ()#('x', 'y', 'size', 'color')
    def __init__(self, x, y, size, color, properties=None):
        super(SVDPoint, self).__init__()
        self.setXYSize(x, y, size)
        self.setBrush(color)
        #if properties is None: properties = {}
        #self.properties = properties
    def setXYSize(self, x, y, size):
        self.setRect(x - size, y - size, x + size, y + size)

#class SVDPixmap(QGraphicsPixmapItem):

class SVDScene(QGraphicsScene):
    def __init__(self, positions, connections):
        QGraphicsScene.__init__(self)
        self.setBackgroundBrush(QColor(50, 50, 50))
        
        # store data from the input matrix
        self.positions = positions
        self.connections = connections
        self.nitems, self.ndim = positions.shape
        self.array = data(positions)
        self.labels = positions.label_list(0)

        # calculate the matrix that projects N-D to 2-D
        self.proj_matrix = np.zeros((self.array.shape[1], 2))
        for idx in (0, 1):
            self.proj_matrix[idx, idx] = 1

        # draw points
        self.locs = np.dot(self.array, self.proj_matrix) * 1000000
        self.points = []
        for idx in xrange(len(self.locs)):
            x, y = self.locs[idx]
            colors = self.array[idx, 0:3]
            colorvec = np.clip(colors*5000+128, 0, 255)
            pen = QColor(*colorvec)
            brush = QColor(*colorvec)
            point = SVDPoint(x, y, .001, pen, brush)
            self.points.append(point)
            self.addItem(point)
        
        xsize = np.max(self.locs[:,0]) - np.min(self.locs[:,0])
        ysize = np.max(self.locs[:,1]) - np.min(self.locs[:,1])
        size = max(xsize, ysize)
        self.bounding_rect = Rect(np.min(self.locs[:,0]), np.min(self.locs[:,1]), size, size)
        
        self.setSceneRect(self.bounding_rect)
        self.setItemIndexMethod(QGraphicsScene.NoIndex)
        #self.setSortCacheEnabled(False)

    def mouseMoveEvent(self, mouseEvent):
        point = mouseEvent.screenPos()
        angle = point.x() / 200
        self.proj_matrix[0,:] = [np.cos(angle), np.sin(angle)]
        self.update_projection()

    def update_projection(self):
        self.locs = np.dot(self.array, self.proj_matrix) * 1000000
        for point, (x, y) in izip(self.points, self.locs):
            point.setXYSize(x, y, 1.0)

class SVDView(QGraphicsView):
    def __init__(self, projection):
        QGraphicsView.__init__(self, projection)
        self.projection = projection   # same as self.scene()
        print self.sceneRect()
        self.fitInView(self.sceneRect(), Qt.KeepAspectRatio)

def main():
    cnet = load_pickle('cnet.pickle')
    svd = cnet.normalized().svd()
    proj = SVDScene(svd.u, cnet)
    view = SVDView(proj)
    view.setGeometry(300, 300, 600, 600)
    view.setWindowTitle("SVDview")
    view.show()
    app.exec_()

if __name__ == '__main__': main()

